# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
# type: ignore
import pathlib
import re
import sys

try:
    from .version import __version__
except ImportError:  # pragma: no cover
    __version__ = "0.0.0.not-installed"
    try:
        from importlib.metadata import version, PackageNotFoundError

        try:
            __version__ = version("pytest-skip-markers")
        except PackageNotFoundError:
            # package is not installed
            pass
    except ImportError:
        try:
            from importlib_metadata import version, PackageNotFoundError

            try:
                __version__ = version("pytest-skip-markers")
            except PackageNotFoundError:
                # package is not installed
                pass
        except ImportError:
            try:
                from pkg_resources import get_distribution, DistributionNotFound

                try:
                    __version__ = get_distribution("pytest-skip-markers").version
                except DistributionNotFound:
                    # package is not installed
                    pass
            except ImportError:
                # pkg resources isn't even available?!
                pass


# Define __version_info__ attribute
VERSION_INFO_REGEX = re.compile(
    r"(?P<major>[\d]+)\.(?P<minor>[\d]+)\.(?P<patch>[\d]+)"
    r"(?:\.dev(?P<commits>[\d]+)\+g(?P<sha>[a-z0-9]+)\.d(?P<date>[\d]+))?"
)
try:
    __version_info__ = tuple(
        int(p) if p.isdigit() else p for p in VERSION_INFO_REGEX.match(__version__).groups() if p
    )
except AttributeError:  # pragma: no cover
    __version_info__ = (-1, -1, -1)
finally:
    del VERSION_INFO_REGEX


# Define some constants
CODE_ROOT_DIR = pathlib.Path(__file__).resolve().parent
IS_WINDOWS = sys.platform.startswith("win")
IS_DARWIN = IS_OSX = sys.platform.startswith("darwin")



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
Platform related utilities.

..
    PYTEST_DONT_REWRITE
"""
import contextlib
import hashlib
import multiprocessing
import os
import pathlib
import platform
import shutil
import subprocess
import sys
from typing import cast

import distro


def is_windows() -> bool:
    """
    Simple function to return if a host is Windows or not.

    :return bool: Return true on Windows
    """
    return sys.platform.startswith("win")


def is_linux() -> bool:
    """
    Simple function to return if a host is Linux or not.

    Note for a proxy minion, we need to return something else
    :return bool: Return true on Linux
    """
    return sys.platform.startswith("linux")


def is_darwin() -> bool:
    """
    Simple function to return if a host is Darwin (macOS) or not.

    :return bool: Return true on Darwin(macOS)
    """
    return sys.platform.startswith("darwin")


def is_sunos() -> bool:
    """
    Simple function to return if host is SunOS or not.

    :return bool: Return true on SunOS
    """
    return sys.platform.startswith("sunos")


def is_smartos() -> bool:
    """
    Simple function to return if host is SmartOS (Illumos) or not.

    :return bool: Return true on SmartOS (Illumos)
    """
    if is_sunos():
        return os.uname()[3].startswith("joyent_")
    return False


def is_freebsd() -> bool:
    """
    Simple function to return if host is FreeBSD or not.

    :return bool: Return true on FreeBSD
    """
    return sys.platform.startswith("freebsd")


def is_netbsd() -> bool:
    """
    Simple function to return if host is NetBSD or not.

    :return bool: Return true on NetBSD
    """
    return sys.platform.startswith("netbsd")


def is_openbsd() -> bool:
    """
    Simple function to return if host is OpenBSD or not.

    :return bool: Return true on OpenBSD
    """
    return sys.platform.startswith("openbsd")


def is_aix() -> bool:
    """
    Simple function to return if host is AIX or not.

    :return bool: Return true on AIX
    """
    return sys.platform.startswith("aix")


def is_aarch64() -> bool:
    """
    Simple function to return if host is AArch64 or not.
    """
    return platform.machine().startswith("aarch64")


def is_photonos() -> bool:
    """
    Simple function to return if host is Photon OS or not.
    """
    osname, _, _ = (x.strip('"').strip("'") for x in distro.linux_distribution())
    return cast(bool, osname == "VMware Photon OS")


def is_spawning_platform() -> bool:
    """
    Returns ``True`` if running on a platform which defaults multiprocessing to spawn.
    """
    return multiprocessing.get_start_method(allow_none=False) == "spawn"


def on_platforms(
    windows: bool = False,
    linux: bool = False,
    darwin: bool = False,
    sunos: bool = False,
    smartos: bool = False,
    freebsd: bool = False,
    netbsd: bool = False,
    openbsd: bool = False,
    aix: bool = False,
    aarch64: bool = False,
    spawning: bool = False,
    photonos: bool = False,
) -> bool:
    """
    Check to see if we're on one of the provided platforms.

    :keyword bool windows: When :py:const:`True`, check if running on Windows.
    :keyword bool linux: When :py:const:`True`, check if running on Linux.
    :keyword bool darwin: When :py:const:`True`, check if running on Darwin.
    :keyword bool sunos: When :py:const:`True`, check if running on SunOS.
    :keyword bool smartos: When :py:const:`True`, check if running on SmartOS.
    :keyword bool freebsd: When :py:const:`True`, check if running on FreeBSD.
    :keyword bool netbsd: When :py:const:`True`, check if running on NetBSD.
    :keyword bool openbsd: When :py:const:`True`, check if running on OpenBSD.
    :keyword bool aix: When :py:const:`True`, check if running on AIX.
    :keyword bool aarch64: When :py:const:`True`, check if running on AArch64.
    :keyword bool spawning:
        When :py:const:`True`, check if running on a platform which defaults
        multiprocessing to spawn
    """
    if windows and is_windows():
        return True

    if linux and is_linux():
        return True

    if darwin and is_darwin():
        return True

    if sunos and is_sunos():
        return True

    if smartos and is_smartos():
        return True

    if freebsd and is_freebsd():
        return True

    if netbsd and is_netbsd():
        return True

    if openbsd and is_openbsd():
        return True

    if aix and is_aix():
        return True

    if aarch64 and is_aarch64():
        return True

    if spawning and is_spawning_platform():
        return True

    if photonos and is_photonos():
        return True

    return False


def is_fips_enabled() -> bool:
    """
    Check is FIPS is enabled.

    :return bool: Return true when enabled
    """
    if pathlib.Path("/etc/system-fips").exists():
        return True
    kernel_fips_enabled_path = pathlib.Path("/proc/sys/crypto/fips_enabled")
    if (
        kernel_fips_enabled_path.exists()
        and kernel_fips_enabled_path.read_text(encoding="utf-8").strip() == "1"
    ):
        return True
    sysctl_path = shutil.which("sysctl")
    if sysctl_path:
        ret = subprocess.run(
            [sysctl_path, "crypto.fips_enabled"],
            check=False,
            shell=False,
            capture_output=True,
            text=True,
        )
        if ret.returncode == 0:
            stripped_output = ret.stdout.strip()
            if (
                stripped_output
                and "=" in stripped_output
                and stripped_output.split("=")[-1].strip() == "1"
            ):
                return True

    with contextlib.suppress(ImportError):
        import cryptography.hazmat.backends.openssl.backend

        if cryptography.hazmat.backends.openssl.backend._fips_enabled:
            return True

    try:
        hashlib.md5()  # nosec
    except ValueError as exc:
        if str(exc) == "[digital envelope routines] unsupported":
            return True
    return False



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
Pytest plugin hooks.
"""
from typing import TYPE_CHECKING

import pytest

import pytestskipmarkers.utils.markers

if TYPE_CHECKING:
    from _pytest.config import Config
    from _pytest.config.argparsing import Parser
    from _pytest.nodes import Item


def pytest_addoption(parser: "Parser") -> None:
    """
    Register argparse-style options and ini-style config values.
    """
    test_selection_group = parser.getgroup("Tests Selection")
    test_selection_group.addoption(
        "--run-destructive",
        action="store_true",
        default=False,
        help="Run destructive tests. These tests can include adding "
        "or removing users from your system for example. "
        "Default: False",
    )
    test_selection_group.addoption(
        "--run-expensive",
        action="store_true",
        default=False,
        help="Run expensive tests. These tests usually involve costs "
        "like for example bootstrapping a cloud VM. "
        "Default: False",
    )


@pytest.hookimpl(tryfirst=True)  # type: ignore[misc]
def pytest_runtest_setup(item: "Item") -> None:
    """
    Fixtures injection based on markers or test skips based on CLI arguments.
    """
    __tracebackhide__ = True
    pytestskipmarkers.utils.markers.evaluate_markers(item)


@pytest.hookimpl(trylast=True)  # type: ignore[misc]
def pytest_configure(config: "Config") -> None:
    """
    Configure the plugin.

    called after command line options have been parsed
    and all plugins and initial conftest files been loaded.
    """
    # Expose the markers we use to pytest CLI
    config.addinivalue_line(
        "markers",
        "destructive_test: Run destructive tests. These tests can include adding "
        "or removing users from your system for example.",
    )
    config.addinivalue_line(
        "markers",
        "expensive_test: Run expensive tests. These tests can include starting resources "
        "which cost money, like VMs, for example.",
    )
    config.addinivalue_line(
        "markers",
        "skip_if_not_root: Skip if the current user is not root on non windows platforms or not "
        "Administrator on windows platforms",
    )
    config.addinivalue_line(
        "markers",
        "skip_if_binaries_missing(*binaries, check_all=True, message=None):"
        "If 'check_all' is True, all binaries must exist."
        "If 'check_all' is False, then only one the passed binaries needs to be found. Usefull when, "
        "for example, passing a list of python interpreter names(python3.5, python3, python), where "
        "only one needs to exist.",
    )
    config.addinivalue_line(
        "markers",
        "requires_network(only_local_network=False): Skip if no networking is set up. "
        "If 'only_local_network' is 'True', only the local network is checked.",
    )
    # Platform Skip Markers
    config.addinivalue_line(
        "markers",
        "skip_on_windows: Skip test on Windows",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_windows: Skip test unless on Windows",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_linux: Skip test on Linux",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_linux: Skip test unless on Linux",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_darwin: Skip test on Darwin",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_darwin: Skip test unless on Darwin",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_sunos: Skip test on SunOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_sunos: Skip test unless on SunOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_smartos: Skip test on SmartOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_smartos: Skip test unless on SmartOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_freebsd: Skip test on FreeBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_freebsd: Skip test unless on FreeBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_netbsd: Skip test on NetBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_netbsd: Skip test unless on NetBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_openbsd: Skip test on OpenBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_openbsd: Skip test unless on OpenBSD",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_aix: Skip test on AIX",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_aix: Skip test unless on AIX",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_aarch64: Skip test on AArch64",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_aarch64: Skip test unless on AArch64",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_photonos: Skip test on PhotonOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_photonos: Skip test unless on PhotonOS",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_spawning_platform: Skip test on spawning platforms",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_spawning_platform: Skip test unless on spawning platforms",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_platforms(windows=False, linux=False, darwin=False, sunos=False, smartos=False, freebsd=False, "
        "netbsd=False, openbsd=False, aix=False, aarch64=False, spawning=False, photonos=False): Pass True to "
        "one or more keywords to get the test skipped.",
    )
    config.addinivalue_line(
        "markers",
        "skip_unless_on_platforms(windows=False, linux=False, darwin=False, sunos=False, smartos=False, freebsd=False, "
        "netbsd=False, openbsd=False, aix=False, aarch64=False, spawning=False, photonos=False): Pass True to "
        "one or more keywords to get the test skipped unless matched.",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_env(envvar, present=True, eq=None, ne=None): Skip test based on environment variables presence "
        "and contents.",
    )
    config.addinivalue_line(
        "markers",
        "skip_on_fips_enabled_platform: Skip test on platforms where FIPS is enabled.",
    )



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
Windows specific functions.

..
    PYTEST_DONT_REWRITE
"""
from typing import List
from typing import Union


try:
    import pywintypes
    import win32api
    import win32net
    import win32security
except ImportError:  # pragma: no cover
    # This is not windows
    pass


def is_admin(name: str) -> bool:
    """
    Is the passed user a member of the Administrators group.

    Args:
        name (str): The name to check

    Returns:
        bool: True if user is a member of the Administrators group, False
        otherwise
    """
    groups = get_user_groups(name, True)

    for group in groups:
        if group in ("S-1-5-32-544", "S-1-5-18"):
            return True

    return False


def get_user_groups(name: str, sid: bool = False) -> List[str]:
    """
    Get the groups to which a user belongs.

    Args:
        name (str): The user name to query
        sid (bool): True will return a list of SIDs, False will return a list of
        group names

    Returns:
        list: A list of group names or sids
    """
    groups = []
    if name.upper() == "SYSTEM":
        # 'win32net.NetUserGetLocalGroups' will fail if you pass in 'SYSTEM'.
        groups = ["SYSTEM"]
    else:
        try:
            groups = win32net.NetUserGetLocalGroups(None, name)
        except (win32net.error, pywintypes.error) as exc:
            # ERROR_ACCESS_DENIED, NERR_DCNotFound, RPC_S_SERVER_UNAVAILABLE
            if exc.winerror in (5, 1722, 2453, 1927, 1355):
                # Try without LG_INCLUDE_INDIRECT flag, because the user might
                # not have permissions for it or something is wrong with DC
                groups = win32net.NetUserGetLocalGroups(None, name, 0)
            else:
                # If this fails, try once more but instead with global groups.
                try:
                    groups = win32net.NetUserGetGroups(None, name)
                except win32net.error as exc:
                    if exc.winerror in (5, 1722, 2453, 1927, 1355):
                        # Try without LG_INCLUDE_INDIRECT flag, because the user might
                        # not have permissions for it or something is wrong with DC
                        groups = win32net.NetUserGetLocalGroups(None, name, 0)
                except pywintypes.error as exc:
                    if exc.winerror in (5, 1722, 2453, 1927, 1355):
                        # Try with LG_INCLUDE_INDIRECT flag, because the user might
                        # not have permissions for it or something is wrong with DC
                        groups = win32net.NetUserGetLocalGroups(None, name, 1)
                    else:
                        raise

    if not sid:
        return groups

    ret_groups = []
    for group in groups:
        ret_groups.append(get_sid_from_name(group))

    return ret_groups


def get_sid_from_name(name: str) -> str:
    """
    This is a tool for getting a sid from a name. The name can be any object.

    Usually a user or a group.

    Args:
        name (str): The name of the user or group for which to get the sid

    Returns:
        str: The corresponding SID
    """
    # If None is passed, use the Universal Well-known SID "Null SID"
    if name is None:
        name = "NULL SID"

    try:
        sid = win32security.LookupAccountName(None, name)[0]
    except pywintypes.error as exc:
        raise Exception(f"User {name} not found: {exc}") from exc

    sidstr: str = win32security.ConvertSidToStringSid(sid)
    return sidstr


def get_current_user(with_domain: bool = True) -> Union[str, bool]:
    """
    Gets the user executing the process.

    Args:

        with_domain (bool):
            ``True`` will prepend the user name with the machine name or domain
            separated by a backslash

    Returns:
        str: The user name
    """
    try:
        user_name: str = win32api.GetUserNameEx(win32api.NameSamCompatible)
        if user_name[-1] == "$":
            # Make the system account easier to identify.
            # Fetch sid so as to handle other language than English
            test_user: str = win32api.GetUserName()
            if test_user == "SYSTEM":
                user_name = "SYSTEM"
            elif get_sid_from_name(test_user) == "S-1-5-18":
                user_name = "SYSTEM"
        elif not with_domain:
            user_name = win32api.GetUserName()
    except pywintypes.error as exc:
        raise Exception(f"Failed to get current user: {exc}") from exc

    if not user_name:
        return False

    return user_name



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
PyTest Markers related utilities.

..
    PYTEST_DONT_REWRITE
"""
import contextlib
import logging
import os
import shutil
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import TYPE_CHECKING
from typing import Union

import pytest

import pytestskipmarkers.utils.platform
import pytestskipmarkers.utils.ports as ports
import pytestskipmarkers.utils.socket as socket

if TYPE_CHECKING:
    from _pytest.nodes import Item

log = logging.getLogger(__name__)


def skip_if_not_root() -> Optional[str]:
    """
    Helper function to check for root/Administrator privileges.

    Returns:
        str: The reason of the skip
    """
    if not pytestskipmarkers.utils.platform.is_windows():
        if os.getuid() != 0:
            return "You must be logged in as root to run this test"
    else:
        from pytestskipmarkers.utils import win_functions

        current_user = win_functions.get_current_user()
        if TYPE_CHECKING:
            assert current_user
        if current_user != "SYSTEM":
            if not win_functions.is_admin(cast(str, current_user)):
                return "You must be logged in as an Administrator to run this test"
    return None


def skip_if_binaries_missing(
    binaries: Union[List[str], Tuple[str, ...]],
    check_all: bool = True,
    reason: Optional[str] = None,
) -> Optional[str]:
    """
    Helper function to check for existing binaries.

    Args:
        binaries (list or tuple):
            Iterator of binaries to check
        check_all (bool):
            If ``check_all`` is ``True``, the default, all binaries must exist.
            If ``check_all`` is ``False``, then only one the passed binaries needs to be found.
            Useful when, for example, passing a list of python interpreter names(python3.5,
            python3, python), where only one needs to exist.
        reason (str):
            The skip reason.

    Returns:
        str: The reason for the skip.
        None: Should not be skipped.
    """
    if check_all is False:
        # We only need one of the passed binaries to exist
        for binary in binaries:
            if shutil.which(binary) is not None:
                break
        else:
            if reason is not None:
                return reason
            return "None of the following binaries was found: {}".format(", ".join(binaries))
    else:
        for binary in binaries:
            if shutil.which(binary) is None:
                if reason is not None:
                    return reason
                return f"The '{binary}' binary was not found"
    log.debug("All binaries found. Searched for: %s", ", ".join(binaries))
    return None


def skip_if_no_local_network() -> Optional[str]:
    """
    Helper function to check for existing local network.

    Returns:
        str: The reason for the skip.
        None: Should not be skipped.
    """
    check_port = ports.get_unused_localhost_port()
    has_local_network = False
    try:
        with contextlib.closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as pubsock:
            pubsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            pubsock.bind(("", check_port))
        has_local_network = True
    except OSError:
        # I wonder if we just have IPV6 support?
        try:
            with contextlib.closing(socket.socket(socket.AF_INET6, socket.SOCK_STREAM)) as pubsock:
                pubsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                pubsock.bind(("", check_port))
            has_local_network = True
        except OSError:
            # Let's continue
            pass
    if has_local_network is False:
        return "No local network was detected"
    return None


def skip_if_no_remote_network() -> Optional[str]:
    """
    Helper function to check for existing remote network(internet).

    Returns:
        str: The reason for the skip.
        None: Should not be skipped.
    """
    if os.environ.get("NO_INTERNET"):
        return "Environment variable NO_INTERNET is set"

    # We are using the google.com DNS records as numerical IPs to avoid
    # DNS look ups which could greatly slow down this check
    has_remote_network = False
    for addr in (
        "172.217.17.14",
        "172.217.16.238",
        "173.194.41.198",
        "173.194.41.199",
        "173.194.41.200",
        "173.194.41.201",
        "173.194.41.206",
        "173.194.41.192",
        "173.194.41.193",
        "173.194.41.194",
        "173.194.41.195",
        "173.194.41.196",
        "173.194.41.197",
        "216.58.201.174",
    ):
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.settimeout(0.25)
            sock.connect((addr, 80))
            sock.close()
            # We connected? Stop the loop
            has_remote_network = True
            break
        except OSError:
            # Let's check the next IP
            continue

    if has_remote_network is False:
        return "No internet network connection was detected"
    return None


def skip_on_env(
    varname: str,
    present: bool = True,
    eq: Optional[str] = None,
    ne: Optional[str] = None,
    reason: Optional[str] = None,
) -> Optional[str]:
    """
    Helper function to check for environment variables.

    If any of the checks match, return the skip reason.

    Args:
        varname(str):
            The environment variable to check
        present(bool):
            When ``True``, skip if variable is present in the environment.
            When ``False``, skip if variable is not present in the environment.
        eq(str):
            Skips when the variable is present in the environment and matches this value.
        ne(str):
            Skips when the variable is present in the environment and does not match this value.
        reason(str):
            The custom reason message to use.

    Returns:
        str: The skip reason
        None: Should not be skipped.
    """
    if eq and ne:
        raise pytest.UsageError("Cannot pass both `eq` and `ne`.")
    if present is False and (eq or ne):
        raise pytest.UsageError("Cannot pass `present=False` and either `eq` or `ne`.")
    if present is False and varname not in os.environ:
        if not reason:
            reason = f"The variable '{varname}' is not present in the environ."
        return reason
    elif present is True and varname in os.environ:
        varname_value = os.environ[varname]
        if eq:
            if varname_value == eq:
                if not reason:
                    reason = f"'{varname}' present in environ and '{varname}=={eq}'"
        elif ne:
            if varname_value != ne:
                if not reason:
                    reason = f"'{varname}' present in environ and '{varname}!={eq}'"
        elif not reason:
            reason = f"The variable '{varname}' is present in the environ."
        return reason
    return None


def evaluate_markers(item: "Item") -> None:
    """
    Fixtures injection based on markers or test skips based on CLI arguments.
    """
    destructive_tests_marker = item.get_closest_marker("destructive_test")
    if destructive_tests_marker is not None:
        if destructive_tests_marker.args or destructive_tests_marker.kwargs:
            raise pytest.UsageError(
                "The 'destructive_test' marker does not accept any arguments or keyword arguments"
            )
        if item.config.getoption("--run-destructive") is False:
            raise pytest.skip.Exception("Destructive tests are disabled", _use_item_location=True)

    expensive_tests_marker = item.get_closest_marker("expensive_test")
    if expensive_tests_marker is not None:
        if expensive_tests_marker.args or expensive_tests_marker.kwargs:
            raise pytest.UsageError(
                "The 'expensive_test' marker does not accept any arguments or keyword arguments"
            )
        if item.config.getoption("--run-expensive") is False:
            raise pytest.skip.Exception("Expensive tests are disabled", _use_item_location=True)

    skip_if_not_root_marker = item.get_closest_marker("skip_if_not_root")
    if skip_if_not_root_marker is not None:
        if skip_if_not_root_marker.args or skip_if_not_root_marker.kwargs:
            raise pytest.UsageError(
                "The 'skip_if_not_root' marker does not accept any arguments or keyword arguments"
            )
        skip_reason = skip_if_not_root()
        if skip_reason:
            raise pytest.skip.Exception(skip_reason, _use_item_location=True)

    skip_if_binaries_missing_marker = item.get_closest_marker("skip_if_binaries_missing")
    if skip_if_binaries_missing_marker is not None:
        binaries = skip_if_binaries_missing_marker.args
        if not binaries:
            raise pytest.UsageError(
                "The 'skip_if_binaries_missing' marker needs at least one binary name to be passed"
            )
        for arg in binaries:
            if not isinstance(arg, str):
                raise pytest.UsageError(
                    "The 'skip_if_binaries_missing' marker only accepts strings as arguments. If you are "
                    "trying to pass multiple binaries, each binary should be an separate argument."
                )
        message = cast(Dict[str, Any], skip_if_binaries_missing_marker.kwargs).pop("message", None)
        if message:
            item.warn(
                pytest.PytestWarning(
                    """Please stop passing 'message="{0}"' and instead pass 'reason="{0}"'""".format(
                        message
                    )
                )
            )
            cast(Dict[str, Any], skip_if_binaries_missing_marker.kwargs)["reason"] = message
        skip_reason = skip_if_binaries_missing(binaries, **skip_if_binaries_missing_marker.kwargs)
        if skip_reason:
            raise pytest.skip.Exception(skip_reason, _use_item_location=True)

    requires_network_marker = item.get_closest_marker("requires_network")
    if requires_network_marker is not None:
        only_local_network = requires_network_marker.kwargs.get("only_local_network", False)
        local_skip_reason = skip_if_no_local_network()
        if local_skip_reason:
            # Since we're only supposed to check local network, and no
            # local network was detected, skip the test
            raise pytest.skip.Exception(local_skip_reason, _use_item_location=True)

        if only_local_network is False:
            remote_skip_reason = skip_if_no_remote_network()
            if remote_skip_reason:
                raise pytest.skip.Exception(remote_skip_reason, _use_item_location=True)

    # Platform Skip Markers
    skip_on_windows_marker = item.get_closest_marker("skip_on_windows")
    if skip_on_windows_marker is not None:
        if skip_on_windows_marker.args:
            raise pytest.UsageError("The skip_on_windows marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_windows_marker.kwargs).pop("reason", None)
        if skip_on_windows_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_windows marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on Windows"
        if pytestskipmarkers.utils.platform.is_windows():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_windows_marker = item.get_closest_marker("skip_unless_on_windows")
    if skip_unless_on_windows_marker is not None:
        if skip_unless_on_windows_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_windows marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_windows_marker.kwargs).pop("reason", None)
        if skip_unless_on_windows_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_windows marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not Windows, skipped"
        if not pytestskipmarkers.utils.platform.is_windows():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_linux_marker = item.get_closest_marker("skip_on_linux")
    if skip_on_linux_marker is not None:
        if skip_on_linux_marker.args:
            raise pytest.UsageError("The skip_on_linux marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_linux_marker.kwargs).pop("reason", None)
        if skip_on_linux_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_linux marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on Linux"
        if pytestskipmarkers.utils.platform.is_linux():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_linux_marker = item.get_closest_marker("skip_unless_on_linux")
    if skip_unless_on_linux_marker is not None:
        if skip_unless_on_linux_marker.args:
            raise pytest.UsageError("The skip_unless_on_linux marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_unless_on_linux_marker.kwargs).pop("reason", None)
        if skip_unless_on_linux_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_linux marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not Linux, skipped"
        if not pytestskipmarkers.utils.platform.is_linux():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_darwin_marker = item.get_closest_marker("skip_on_darwin")
    if skip_on_darwin_marker is not None:
        if skip_on_darwin_marker.args:
            raise pytest.UsageError("The skip_on_darwin marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_darwin_marker.kwargs).pop("reason", None)
        if skip_on_darwin_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_darwin marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on Darwin"
        if pytestskipmarkers.utils.platform.is_darwin():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_darwin_marker = item.get_closest_marker("skip_unless_on_darwin")
    if skip_unless_on_darwin_marker is not None:
        if skip_unless_on_darwin_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_darwin marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_darwin_marker.kwargs).pop("reason", None)
        if skip_unless_on_darwin_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_darwin marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not Darwin, skipped"
        if not pytestskipmarkers.utils.platform.is_darwin():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_sunos_marker = item.get_closest_marker("skip_on_sunos")
    if skip_on_sunos_marker is not None:
        if skip_on_sunos_marker.args:
            raise pytest.UsageError("The skip_on_sunos marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_sunos_marker.kwargs).pop("reason", None)
        if skip_on_sunos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_sunos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on SunOS"
        if pytestskipmarkers.utils.platform.is_sunos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_sunos_marker = item.get_closest_marker("skip_unless_on_sunos")
    if skip_unless_on_sunos_marker is not None:
        if skip_unless_on_sunos_marker.args:
            raise pytest.UsageError("The skip_unless_on_sunos marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_unless_on_sunos_marker.kwargs).pop("reason", None)
        if skip_unless_on_sunos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_sunos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not SunOS, skipped"
        if not pytestskipmarkers.utils.platform.is_sunos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_smartos_marker = item.get_closest_marker("skip_on_smartos")
    if skip_on_smartos_marker is not None:
        if skip_on_smartos_marker.args:
            raise pytest.UsageError("The skip_on_smartos marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_smartos_marker.kwargs).pop("reason", None)
        if skip_on_smartos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_smartos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on SmartOS"
        if pytestskipmarkers.utils.platform.is_smartos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_smartos_marker = item.get_closest_marker("skip_unless_on_smartos")
    if skip_unless_on_smartos_marker is not None:
        if skip_unless_on_smartos_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_smartos marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_smartos_marker.kwargs).pop("reason", None)
        if skip_unless_on_smartos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_smartos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not SmartOS, skipped"
        if not pytestskipmarkers.utils.platform.is_smartos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_freebsd_marker = item.get_closest_marker("skip_on_freebsd")
    if skip_on_freebsd_marker is not None:
        if skip_on_freebsd_marker.args:
            raise pytest.UsageError("The skip_on_freebsd marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_freebsd_marker.kwargs).pop("reason", None)
        if skip_on_freebsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_freebsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on FreeBSD"
        if pytestskipmarkers.utils.platform.is_freebsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_freebsd_marker = item.get_closest_marker("skip_unless_on_freebsd")
    if skip_unless_on_freebsd_marker is not None:
        if skip_unless_on_freebsd_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_freebsd marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_freebsd_marker.kwargs).pop("reason", None)
        if skip_unless_on_freebsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_freebsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not FreeBSD, skipped"
        if not pytestskipmarkers.utils.platform.is_freebsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_netbsd_marker = item.get_closest_marker("skip_on_netbsd")
    if skip_on_netbsd_marker is not None:
        if skip_on_netbsd_marker.args:
            raise pytest.UsageError("The skip_on_netbsd marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_netbsd_marker.kwargs).pop("reason", None)
        if skip_on_netbsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_netbsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on NetBSD"
        if pytestskipmarkers.utils.platform.is_netbsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_netbsd_marker = item.get_closest_marker("skip_unless_on_netbsd")
    if skip_unless_on_netbsd_marker is not None:
        if skip_unless_on_netbsd_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_netbsd marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_netbsd_marker.kwargs).pop("reason", None)
        if skip_unless_on_netbsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_netbsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not NetBSD, skipped"
        if not pytestskipmarkers.utils.platform.is_netbsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_openbsd_marker = item.get_closest_marker("skip_on_openbsd")
    if skip_on_openbsd_marker is not None:
        if skip_on_openbsd_marker.args:
            raise pytest.UsageError("The skip_on_openbsd marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_openbsd_marker.kwargs).pop("reason", None)
        if skip_on_openbsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_openbsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on OpenBSD"
        if pytestskipmarkers.utils.platform.is_openbsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_openbsd_marker = item.get_closest_marker("skip_unless_on_openbsd")
    if skip_unless_on_openbsd_marker is not None:
        if skip_unless_on_openbsd_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_openbsd marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_openbsd_marker.kwargs).pop("reason", None)
        if skip_unless_on_openbsd_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_openbsd marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not OpenBSD, skipped"
        if not pytestskipmarkers.utils.platform.is_openbsd():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_aix_marker = item.get_closest_marker("skip_on_aix")
    if skip_on_aix_marker is not None:
        if skip_on_aix_marker.args:
            raise pytest.UsageError("The skip_on_aix marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_aix_marker.kwargs).pop("reason", None)
        if skip_on_aix_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_aix marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on AIX"
        if pytestskipmarkers.utils.platform.is_aix():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_aix_marker = item.get_closest_marker("skip_unless_on_aix")
    if skip_unless_on_aix_marker is not None:
        if skip_unless_on_aix_marker.args:
            raise pytest.UsageError("The skip_unless_on_aix marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_unless_on_aix_marker.kwargs).pop("reason", None)
        if skip_unless_on_aix_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_aix marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not AIX, skipped"
        if not pytestskipmarkers.utils.platform.is_aix():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_aarch64_marker = item.get_closest_marker("skip_on_aarch64")
    if skip_on_aarch64_marker is not None:
        if skip_on_aarch64_marker.args:
            raise pytest.UsageError("The skip_on_aarch64 marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_aarch64_marker.kwargs).pop("reason", None)
        if skip_on_aarch64_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_aarch64 marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on AArch64"
        if pytestskipmarkers.utils.platform.is_aarch64():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_aarch64_marker = item.get_closest_marker("skip_unless_on_aarch64")
    if skip_unless_on_aarch64_marker is not None:
        if skip_unless_on_aarch64_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_aarch64 marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_aarch64_marker.kwargs).pop("reason", None)
        if skip_unless_on_aarch64_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_aarch64 marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not AArch64, skipped"
        if not pytestskipmarkers.utils.platform.is_aarch64():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_spawning_platform_marker = item.get_closest_marker("skip_on_spawning_platform")
    if skip_on_spawning_platform_marker is not None:
        if skip_on_spawning_platform_marker.args:
            raise pytest.UsageError(
                "The skip_on_spawning_platform marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_on_spawning_platform_marker.kwargs).pop("reason", None)
        if skip_on_spawning_platform_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_spawning_platform marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on spawning platforms"
        if pytestskipmarkers.utils.platform.is_spawning_platform():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_spawning_platform_marker = item.get_closest_marker(
        "skip_unless_on_spawning_platform"
    )
    if skip_unless_on_spawning_platform_marker is not None:
        if skip_unless_on_spawning_platform_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_spawning_platform marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_spawning_platform_marker.kwargs).pop(
            "reason", None
        )
        if skip_unless_on_spawning_platform_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_spawning_platform marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform does not default multiprocessing to spawn, skipped"
        if not pytestskipmarkers.utils.platform.is_spawning_platform():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_photonos_marker = item.get_closest_marker("skip_on_photonos")
    if skip_on_photonos_marker is not None:
        if skip_on_photonos_marker.args:
            raise pytest.UsageError("The skip_on_photonos marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_photonos_marker.kwargs).pop("reason", None)
        if skip_on_photonos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_photonos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on PhotonOS"
        if pytestskipmarkers.utils.platform.is_photonos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_unless_on_photonos_marker = item.get_closest_marker("skip_unless_on_photonos")
    if skip_unless_on_photonos_marker is not None:
        if skip_unless_on_photonos_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_photonos marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_photonos_marker.kwargs).pop("reason", None)
        if skip_unless_on_photonos_marker.kwargs:
            raise pytest.UsageError(
                "The skip_unless_on_photonos marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Platform is not PhotonOS, skipped"
        if not pytestskipmarkers.utils.platform.is_photonos():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_fips_enabled_platform_marker = item.get_closest_marker("skip_on_fips_enabled_platform")
    if skip_on_fips_enabled_platform_marker is not None:
        if skip_on_fips_enabled_platform_marker.args:
            raise pytest.UsageError(
                "The skip_on_fips_enabled_platform marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_on_fips_enabled_platform_marker.kwargs).pop(
            "reason", None
        )
        if skip_on_fips_enabled_platform_marker.kwargs:
            raise pytest.UsageError(
                "The skip_on_fips_enabled_platform marker only accepts 'reason' as a keyword argument."
            )
        if reason is None:
            reason = "Skipped on FIPS enabled platform"
        if pytestskipmarkers.utils.platform.is_fips_enabled():
            raise pytest.skip.Exception(reason, _use_item_location=True)

    skip_on_platforms_marker = item.get_closest_marker("skip_on_platforms")
    if skip_on_platforms_marker is not None:
        if skip_on_platforms_marker.args:
            raise pytest.UsageError("The skip_on_platforms marker does not accept any arguments")
        reason = cast(Dict[str, Any], skip_on_platforms_marker.kwargs).pop("reason", None)
        if not skip_on_platforms_marker.kwargs:
            raise pytest.UsageError(
                "Pass at least one platform to skip_on_platforms as a keyword argument"
            )
        if not any(skip_on_platforms_marker.kwargs.values()):
            raise pytest.UsageError(
                "Pass at least one platform with a True value to skip_on_platforms as a keyword argument"
            )
        if reason is None:
            reason = "Skipped on platform match"
        try:
            if pytestskipmarkers.utils.platform.on_platforms(**skip_on_platforms_marker.kwargs):
                raise pytest.skip.Exception(reason, _use_item_location=True)
        except TypeError as exc:
            raise pytest.UsageError(f"Passed an invalid platform to skip_on_platforms: {exc}")

    skip_unless_on_platforms_marker = item.get_closest_marker("skip_unless_on_platforms")
    if skip_unless_on_platforms_marker is not None:
        if skip_unless_on_platforms_marker.args:
            raise pytest.UsageError(
                "The skip_unless_on_platforms marker does not accept any arguments"
            )
        reason = cast(Dict[str, Any], skip_unless_on_platforms_marker.kwargs).pop("reason", None)
        if not skip_unless_on_platforms_marker.kwargs:
            raise pytest.UsageError(
                "Pass at least one platform to skip_unless_on_platforms as a keyword argument"
            )
        if not any(skip_unless_on_platforms_marker.kwargs.values()):
            raise pytest.UsageError(
                "Pass at least one platform with a True value to skip_unless_on_platforms as a keyword argument"
            )
        if reason is None:
            reason = "Platform(s) do not match, skipped"
        try:
            if not pytestskipmarkers.utils.platform.on_platforms(
                **skip_unless_on_platforms_marker.kwargs
            ):
                raise pytest.skip.Exception(reason, _use_item_location=True)
        except TypeError as exc:
            raise pytest.UsageError(
                f"Passed an invalid platform to skip_unless_on_platforms: {exc}"
            )

    skip_on_env_marker = item.get_closest_marker("skip_on_env")
    if skip_on_env_marker is not None:
        args = list(skip_on_env_marker.args)
        if not args:
            raise pytest.UsageError(
                "The 'skip_on_env' marker needs at least one argument to be passed, the environment "
                "variable name."
            )
        envvar = args.pop(0)
        if args:
            raise pytest.UsageError(
                "The 'skip_on_env' only accepts one argument, the environment variable name."
            )
        if not isinstance(envvar, str):
            raise pytest.UsageError("The environment variable argument must be a string.")

        kwargs = cast(Dict[str, Any], skip_on_env_marker.kwargs).copy()
        present = kwargs.pop("present", True)
        eq = kwargs.pop("eq", None)
        ne = kwargs.pop("ne", None)
        reason = kwargs.pop("reason", None)
        if kwargs:
            raise pytest.UsageError(
                "The 'skip_on_env' marker only accepts 'present', 'eq', 'ne' and 'reason' as "
                "keyword arguments."
            )
        skip_reason = skip_on_env(envvar, present=present, eq=eq, ne=ne, reason=reason)
        if skip_reason:
            raise pytest.skip.Exception(skip_reason, _use_item_location=True)



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
Namespaced ``socket`` module.

This module's sole purpose is to have the standard library :py:mod:`socket` module
functions under a different namespace to be used in pytest-skip-markers so that
projects using it, which need to mock :py:mod:`socket` functions, don't influence
the pytest-skip-markers run time behavior.
"""
from socket import *  # pylint: disable=wildcard-import,unused-wildcard-import



# Copyright 2021-2024 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
#
"""
Ports related utility functions.
"""
import contextlib
import logging
from typing import Set

import pytest

from pytestskipmarkers.utils import socket

log = logging.getLogger(__name__)


def get_unused_localhost_port(use_cache: bool = False) -> int:
    """
    Return a random unused port on localhost.

    :keyword bool use_cache:
        If ``use_cache`` is ``True``, consecutive calls to this function will never return the cached port.
    """
    if not isinstance(use_cache, bool):
        raise pytest.UsageError(
            f"The value of 'use_cache' needs to be an boolean, not {type(use_cache)}"
        )

    with contextlib.closing(socket.socket(family=socket.AF_INET, type=socket.SOCK_STREAM)) as usock:
        usock.bind(("127.0.0.1", 0))
        port: int = usock.getsockname()[1]

    if use_cache:
        try:
            cached_ports = get_unused_localhost_port.__cached_ports__  # type: ignore[attr-defined]
        except AttributeError:
            cached_ports = get_unused_localhost_port.__cached_ports__ = set()  # type: ignore[attr-defined]
        if port in cached_ports:
            return get_unused_localhost_port(use_cache=use_cache)
        cached_ports.add(port)

    return port


def get_connectable_ports(ports: Set[int]) -> Set[int]:
    """
    Returns a set of the ports where connection was successful.

    :param ~collections.abc.Iterable ports: An iterable of ports to try and connect to
    :rtype: set
    :return: Returns a set of the ports where connection was successful
    """
    connectable_ports = set()
    ports = set(ports)

    for port in set(ports):
        with contextlib.closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
            conn = sock.connect_ex(("localhost", port))
            try:
                if conn == 0:
                    log.debug("Port %s is connectable!", port)
                    connectable_ports.add(port)
                    sock.shutdown(socket.SHUT_RDWR)
            except OSError:
                continue
    return connectable_ports
